==> moonlight-1.20-2.16.15-fabric-sources.jar: net/mehvahdjukaar/moonlight/api/platform/fabric/RegistryQueue.java <==
package net.mehvahdjukaar.moonlight.api.platform.fabric;

import com.mojang.serialization.Lifecycle;
import net.fabricmc.fabric.api.client.rendering.v1.BuiltinItemRendererRegistry;
import net.mehvahdjukaar.moonlight.api.client.ICustomItemRendererProvider;
import net.mehvahdjukaar.moonlight.api.misc.RegSupplier;
import net.mehvahdjukaar.moonlight.api.misc.Registrator;
import net.mehvahdjukaar.moonlight.api.platform.PlatHelper;
import net.mehvahdjukaar.moonlight.api.platform.RegHelper;
import net.minecraft.class_1935;
import net.minecraft.class_2378;
import net.minecraft.class_2385;
import net.minecraft.class_2960;
import net.minecraft.class_5321;
import net.minecraft.class_6880;
import net.minecraft.class_7923;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;
import java.util.function.Supplier;

public class RegistryQueue<T> {
    private final class_5321<? extends class_2378<T>> registry;
    private final List<EntryWrapper<? extends T, T>> entries = new ArrayList<>();
    private final List<Consumer<Registrator<T>>> batchRegistration = new ArrayList<>();

    public RegistryQueue(class_5321<? extends class_2378<T>> registry) {
        this.registry = registry;
    }

    public class_5321<? extends class_2378<T>> getRegistry() {
        return registry;
    }

    public <A extends T> EntryWrapper<A, T> add(Supplier<A> factory, class_2960 name) {
        EntryWrapper<A, T> wrapper = new EntryWrapper<>(name, factory, registry);
        entries.add(wrapper);
        return wrapper;
    }

    public void add(Consumer<Registrator<T>> eventListener) {
        batchRegistration.add(eventListener);
    }

    void initializeEntries() {
        entries.forEach(EntryWrapper::initialize);
        batchRegistration.forEach(e -> e.accept((n, s) -> RegHelper.registerAsync(n, () -> s, registry)));
    }


    static class EntryWrapper<T extends R, R> implements RegSupplier<T> {
        private final class_2960 id;
        private class_5321<? extends class_2378<R>> registryKey;
        private Supplier<T> regSupplier;
        private T entry;
        private class_6880<T> holder;


        public EntryWrapper(class_2960 id, Supplier<T> factory, class_5321<? extends class_2378<R>> registry) {
            this.regSupplier = factory;
            this.id = id;
            this.registryKey = registry;
        }


        @Override
        public T get() {
            return entry;
        }

        @Override
        public class_2960 getId() {
            return id;
        }

        @Override
        public class_5321<T> getKey() {
            return holder.method_40230().get();
        }

        @Override
        public class_6880<T> getHolder() {
            return holder;
        }

        void initialize() {
            this.holder = ((class_2385) class_7923.field_41167.method_10223(registryKey.method_29177()))
                    .method_10272(class_5321.method_29179(registryKey, id), regSupplier.get(), Lifecycle.stable());
            this.entry = this.holder.comp_349();
            regSupplier = null;
            registryKey = null;

            if (PlatHelper.getPhysicalSide().isClient() && entry instanceof ICustomItemRendererProvider pr) {
                class_1935 il = (class_1935) entry;
                if (BuiltinItemRendererRegistry.INSTANCE.get(il) == null) {
                    BuiltinItemRendererRegistry.INSTANCE.register(il,
                            (BuiltinItemRendererRegistry.DynamicItemRenderer) pr.getRendererFactory().get());
                } else {
                    if (PlatHelper.isDev()) throw new AssertionError();
                }
            }
        }

    }
}

==> moonlight-1.21-2.25.0-fabric-sources.jar: net/mehvahdjukaar/moonlight/api/platform/fabric/RegistryQueue.java <==
package net.mehvahdjukaar.moonlight.api.platform.fabric;

import com.mojang.serialization.Lifecycle;
import net.fabricmc.fabric.api.client.rendering.v1.BuiltinItemRendererRegistry;
import net.mehvahdjukaar.moonlight.api.client.ICustomItemRendererProvider;
import net.mehvahdjukaar.moonlight.api.misc.RegSupplier;
import net.mehvahdjukaar.moonlight.api.misc.Registrator;
import net.mehvahdjukaar.moonlight.api.platform.PlatHelper;
import net.mehvahdjukaar.moonlight.api.platform.RegHelper;
import net.minecraft.class_1935;
import net.minecraft.class_2378;
import net.minecraft.class_2385;
import net.minecraft.class_2960;
import net.minecraft.class_5321;
import net.minecraft.class_6880;
import net.minecraft.class_7923;
import net.minecraft.class_9248;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;
import java.util.function.Supplier;

public class RegistryQueue<T> {
    private final class_5321<? extends class_2378<T>> registry;
    private final List<EntryWrapper<? extends T, T>> entries = new ArrayList<>();
    private final List<Consumer<Registrator<T>>> batchRegistration = new ArrayList<>();

    public RegistryQueue(class_5321<? extends class_2378<T>> registry) {
        this.registry = registry;
    }

    public class_5321<? extends class_2378<T>> getRegistry() {
        return registry;
    }

    public <A extends T> EntryWrapper<A, T> add(Supplier<A> factory, class_2960 name) {
        EntryWrapper<A, T> wrapper = new EntryWrapper<>(name, factory, registry);
        entries.add(wrapper);
        return wrapper;
    }

    public void add(Consumer<Registrator<T>> eventListener) {
        batchRegistration.add(eventListener);
    }

    void initializeEntries() {
        entries.forEach(EntryWrapper::initialize);
        batchRegistration.forEach(e -> e.accept((n, s) -> RegHelper.registerAsync(n, () -> s, registry)));
    }


    static class EntryWrapper<T extends R, R> implements RegSupplier<T> {
        private final class_5321<R> id;
        private class_5321<? extends class_2378<R>> registryKey;
        private Supplier<T> regSupplier;
        private T entry;
        private class_6880<T> holder;


        public EntryWrapper(class_2960 id, Supplier<T> factory, class_5321<? extends class_2378<R>> registry) {
            this.regSupplier = factory;
            this.id = class_5321.method_29179(registry, id);
            this.registryKey = registry;
        }


        @Override
        public T get() {
            return entry;
        }

        @Override
        public class_2960 getId() {
            return id.method_29177();
        }

        @Override
        public class_5321<R> getKey() {
            return id;
        }

        @Override
        public class_6880<T> getHolder() {
            return holder;
        }

        void initialize() {
            class_2385 writableRegistry = (class_2385) class_7923.field_41167.method_10223(registryKey.method_29177());
            if(writableRegistry == null) throw new IllegalStateException("Registry not found: " + registryKey.method_29177());
            this.holder = writableRegistry.method_10272(id, regSupplier.get(), class_9248.field_49136);
            this.entry = this.holder.comp_349();
            regSupplier = null;
            registryKey = null;

            if (PlatHelper.getPhysicalSide().isClient() && entry instanceof ICustomItemRendererProvider pr) {
                class_1935 il = (class_1935) entry;
                if (BuiltinItemRendererRegistry.INSTANCE.get(il) == null) {
                    BuiltinItemRendererRegistry.INSTANCE.register(il,
                            (BuiltinItemRendererRegistry.DynamicItemRenderer) pr.getRendererFactory().get());
                } else {
                    if (PlatHelper.isDev()) throw new AssertionError();
                }
            }
        }

    }
}
